package com.sun.mojarra.scales.component;

import com.sun.mojarra.scales.util.CssUtil;
import com.sun.mojarra.scales.util.RenderingHelper;
import com.sun.mojarra.scales.util.ScalesUtil;
import com.sun.mojarra.scales.util.TemplateHandlers;
import com.sun.mojarra.scales.util.YuiConstants;
import java.io.IOException;
import java.util.Map;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 *
 * @author Jason Lee
 */
public class ColorPicker extends UIInput {

    public static final String COMPONENT_FAMILY = "com.sun.mojarra.scales.component.ColorPicker";
    public static final String RENDERER_TYPE = COMPONENT_FAMILY;
    public static final String COMPONENT_TYPE = COMPONENT_FAMILY;

    // Component Properties
    private boolean showControls;
    private boolean showHexControls;
    private boolean showHexSummary;
    private boolean showHsvControls;
    private boolean showRgbControls;
    private boolean showWebSafe;
    private String backgroundColor;
    private String style;
    private String styleClass;
    // Internal variables
    private Object[] _state = null;

    public ColorPicker() {
        super();
        setRendererType(RENDERER_TYPE);

        FacesContext context = FacesContext.getCurrentInstance();
        if (context != null) {
            String baseUrl = context.getExternalContext().getRequestContextPath();
            Links.addCss(Links.EXTERNAL_STYLESHEETS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.CSS_COLOR_PICKER));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_SLIDER));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_COLORPICKER));
            CssUtil.linkColorPickerOverrides(context);

            setValue("000000");
        }
    }

    @Override
    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    public String getBackgroundColor() {
        return ScalesUtil.getPropertyValue(this, backgroundColor, "backgroundColor", "eeeeee");
    }

    public void setBackgroundColor(String backgroundColor) {
        this.backgroundColor = backgroundColor;
    }

    public boolean getShowControls() {
        return ScalesUtil.getPropertyValue(this, showControls, "showControls", Boolean.TRUE);
    }

    public void setShowControls(boolean showControls) {
        this.showControls = showControls;
    }

    public boolean getShowHexControls() {
        return ScalesUtil.getPropertyValue(this, showHexControls, "showHexControls", Boolean.TRUE);
    }

    public void setShowHexControls(boolean showHexControls) {
        this.showHexControls = showHexControls;
    }

    public boolean getShowHexSummary() {
        return ScalesUtil.getPropertyValue(this, showHexSummary, "showHexSummary", Boolean.TRUE);
    }

    public void setShowHexSummary(boolean showHexSummary) {
        this.showHexSummary = showHexSummary;
    }

    public boolean getShowHsvControls() {
        return ScalesUtil.getPropertyValue(this, showHsvControls, "showHsvControls", Boolean.FALSE);
    }

    public void setShowHsvControls(boolean showHsvControls) {
        this.showHsvControls = showHsvControls;
    }

    public boolean getShowRgbControls() {
        return ScalesUtil.getPropertyValue(this, showRgbControls, "showRgbControls", Boolean.TRUE);
    }

    public void setShowRgbControls(boolean showRgbControls) {
        this.showRgbControls = showRgbControls;
    }

    public boolean getShowWebSafe() {
        return ScalesUtil.getPropertyValue(this, showWebSafe, "showWebSafe", Boolean.TRUE);
    }

    public void setShowWebSafe(boolean showWebSafe) {
        this.showWebSafe = showWebSafe;
    }

    public String getStyle() {
        return ScalesUtil.getPropertyValue(this, style, "style", "");
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        return ScalesUtil.getPropertyValue(this, styleClass, "styleClass", "");
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    @Override
    public Object saveState(FacesContext context) {
        if (_state == null) {
            _state = new Object[10];
        }
        _state[0] = super.saveState(context);
        _state[1] = showControls;
        _state[2] = showHexControls;
        _state[3] = showHexSummary;
        _state[4] = showHsvControls;
        _state[5] = showRgbControls;
        _state[6] = showWebSafe;
        _state[7] = backgroundColor;
        _state[8] = style;
        _state[9] = styleClass;
        return _state;
    }

    @Override
    public void restoreState(FacesContext context, Object state) {
        this._state = (Object[]) state;
        super.restoreState(context, this._state[0]);
        showControls = (Boolean) _state[1];
        showHexControls = (Boolean) _state[2];
        showHexSummary = (Boolean) _state[3];
        showHsvControls = (Boolean) _state[4];
        showRgbControls = (Boolean) _state[5];
        showWebSafe = (Boolean) _state[6];
        backgroundColor = (String) _state[7];
        style = (String) _state[8];
        styleClass = (String) _state[9];
    }

    @Override
    public void encodeBegin(FacesContext context) throws IOException {
        super.encodeBegin(context);

        // suppress rendering if "rendered" property on the component is false.
        if (!isRendered()) {
            return;
        }
        ResponseWriter writer = context.getResponseWriter();
        String clientId = getClientId(context);

        writer.startElement("span", this);
        writer.writeAttribute("class", getStyleClass(), "class");
        writer.writeAttribute("style", getStyle(), "style");

        RenderingHelper.outputElement(writer, this, "div",
                                      new String[][]{{"id", clientId}, {"class", getStyleClass()},
                                                     {"style", "position: relative; padding: 6px; background-color: #" + getBackgroundColor() + "; width: 420px; height:220px; " + getStyle() + "}"}});
        RenderingHelper.writeOnDomReady(writer, "createColorPicker('" + clientId + "', {" +
                buildConfig(context) + "});");
    }

    protected String buildConfig(FacesContext context) {
        StringBuilder builder = new StringBuilder();
        String baseUrl = TemplateHandlers.getStaticResourceUrl(context, "");
        String value = (String) getValue();

        
        ScalesUtil.appendToConfig(builder, "showcontrols", getShowControls(), false, false);
        ScalesUtil.appendToConfig(builder, "showhexcontrols", getShowHexControls(), false, true);
        ScalesUtil.appendToConfig(builder, "showhexsummary", getShowHexSummary(), false, true);
        ScalesUtil.appendToConfig(builder, "showhsvcontrols", getShowHsvControls(), false, true);
        ScalesUtil.appendToConfig(builder, "showrgbcontrols", getShowRgbControls(), false, true);
        ScalesUtil.appendToConfig(builder, "showwebsafe", getShowWebSafe(), false, true);
        ScalesUtil.appendToConfig(builder, "images", "{PICKER_THUMB: '" + baseUrl + "yui/colorpicker/assets/picker_thumb.png',HUE_THUMB: '" +baseUrl + "yui/colorpicker/assets/hue_thumb.png'}", false, true);

        if (value != null) {
            addRgbToConfig(builder, value);
        }

        return builder.toString();
    }

    @Override
    // TODO: <!decode   convertDate(dateFormat="#{param.$this{clientId}_dateFormat}") />
    public void decode(FacesContext facesContext) {
        Map requestMap = facesContext.getExternalContext().getRequestParameterMap();
        setSubmittedValue((String) requestMap.get("yui-picker-hex"));
    }

    protected void addRgbToConfig(StringBuilder builder, String color) {
        String red;
        String green;
        String blue;

        if (color == null) {
            return;
        }
        if (color.startsWith("#")) {
            color = color.substring(1);
        }
        if (color.length() == 3) {
            red = color.substring(0, 1) + color.substring(0, 1);
            green = color.substring(1, 2) + color.substring(1, 2);
            blue = color.substring(2, 3) + color.substring(2, 3);
        } else {
            red = color.substring(0, 2);
            green = color.substring(2, 4);
            blue = color.substring(4, 6);
        }
        ScalesUtil.appendToConfig(builder, "red", Integer.parseInt(red, 16), false, true);
        ScalesUtil.appendToConfig(builder, "green", Integer.parseInt(green, 16), false, true);
        ScalesUtil.appendToConfig(builder, "blue", Integer.parseInt(blue, 16), false, true);
    }
}