/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
/**
 * 
 */
package com.sun.mojarra.scales.component;


import com.sun.mojarra.scales.util.CssUtil;
import com.sun.mojarra.scales.util.RenderingHelper;
import com.sun.mojarra.scales.util.ScalesUtil;
import com.sun.mojarra.scales.util.YuiConstants;
import java.io.IOException;
import java.util.Map;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 * @author Jason Lee
 *
 */
public class HtmlEditor extends UIInput {

    public static final String COMPONENT_FAMILY = "com.sun.mojarra.scales.HtmlEditor";
    public static final String COMPONENT_TYPE = COMPONENT_FAMILY;
    public static final String RENDERER_TYPE = COMPONENT_FAMILY;
    private Object[] _state = null;
    private String width;
    private String height;
    private boolean domPath;
    private boolean animate;

    public HtmlEditor() {
        setRendererType(RENDERER_TYPE);

        FacesContext context = FacesContext.getCurrentInstance();
        if (context != null) {
            String baseUrl = context.getExternalContext().getRequestContextPath();
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_CONTAINER));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_MENU));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_BUTTON));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_EDITOR));
            CssUtil.linkContainerOverrides(context);
            CssUtil.linkButtonOverrides(context);
            CssUtil.linkMenuOverrides(context);
            CssUtil.linkEditorOverrides(context);
        }
    }

    @Override
    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    @Override
    public void encodeBegin(FacesContext context) throws IOException {
        super.encodeBegin(context);

        // suppress rendering if "rendered" property on the component is false.
        if (!isRendered()) {
            return;
        }

        ResponseWriter writer = context.getResponseWriter();
        writer.startElement("div", this);
        writer.writeAttribute("class", "yui-skin-sam", "class");
        writer.startElement("textarea", this);
        writer.writeAttribute("id", getClientId(context), "id");
        writer.writeAttribute("name", getClientId(context), "name");
        writer.write((String)getValue());
        writer.endElement("textarea");
        writer.endElement("div");
        RenderingHelper.writeOnDomReady(writer, "createEditor({" + buildConfig(context) + "});");
    }

    protected String buildConfig(FacesContext context) {
        StringBuilder builder = new StringBuilder();

        builder.append ("clientId: '")
                .append(getClientId(context))
                .append("', height: '")
                .append(getHeight())
                .append("', width: '")
                .append(getWidth())
                .append("', domPath : ")
                .append(getDomPath())
                .append(", animate: ")
                .append(getAnimate());

        return builder.toString();
    }

    @Override
    public void decode(FacesContext facesContext) {
        Map requestMap = facesContext.getExternalContext().getRequestParameterMap();
        String clientId = getClientId(facesContext);

        String string_submit_val = ((String) requestMap.get(clientId));

        setSubmittedValue(string_submit_val);
    }

    public boolean getAnimate() {
        return ScalesUtil.getPropertyValue(this, animate, "animate", Boolean.FALSE);
    }

    public void setAnimate(boolean animate) {
        this.animate = animate;
    }

    public boolean getDomPath() {
        return ScalesUtil.getPropertyValue(this, domPath, "domPath", Boolean.FALSE);
    }

    public void setDomPath(boolean domPath) {
        this.domPath = domPath;
    }

    public String getHeight() {
        return ScalesUtil.getPropertyValue(this, height, "height", "300px");
    }

    public void setHeight(String height) {
        this.height = height;
    }

    public String getWidth() {
        return ScalesUtil.getPropertyValue(this, width, "width", "450px");
    }

    public void setWidth(String width) {
        this.width = width;
    }


    @Override
    public void restoreState(FacesContext _context, Object _state) {
        this._state = (Object[]) _state;
        super.restoreState(_context, this._state[0]);
        width = (String) this._state[1];
        height = (String) this._state[2];
        domPath = (Boolean) this._state[3];
        animate = (Boolean) this._state[4];
    }

    @Override
    public Object saveState(FacesContext _context) {
        if (_state == null) {
            _state = new Object[5];
        }
        _state[0] = super.saveState(_context);
        _state[1] = width;
        _state[2] = height;
        _state[3] = domPath;
        _state[4] = animate;

        return _state;
    }
}