/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.sun.mojarra.scales.renderer;

import com.sun.mojarra.scales.component.Links;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

/**
 *
 * @author Jason CTR Lee
 */
public class LinksRenderer extends Renderer {
    
    public LinksRenderer() {
    }

    @Override
    @SuppressWarnings("unchecked")
    public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
        if (context == null) {
            throw new NullPointerException("param 'context' is null");
        }
        if (component == null) {
            throw new NullPointerException("param 'component' is null");
        }

        // suppress rendering if "rendered" property on the component is false.
        if (!component.isRendered()) {
            return;
        }
        
        if (!(component instanceof Links)) {
            throw new IllegalStateException ("Was expecting Links.  Found " +
                    component.getClass().getName());
        }
        
        Links links = (Links)component;

        Set<String> scripts = Links.getScripts(Links.EXTERNAL_SCRIPTS);
        Set<String> onReady = Links.getScripts(Links.DOM_READY_SCRIPTS);
        Set<String> externalCss = Links.getCss(Links.EXTERNAL_STYLESHEETS);
        Set<String> inlineCss = Links.getCss(Links.INLINE_STYLESHEET);
        Map<String, String> tags = Links.getMeta();
        // If nothing has been registered, bail out early
        if ((scripts.size() > 0) || (onReady.size() > 0) || (externalCss.size() > 0) || (tags.size() > 0) || (inlineCss.size() > 0)) {
            // suppress rendering if "rendered" property on the component is false.

            ResponseWriter writer = context.getResponseWriter();

            if (tags != null) {
                for (Map.Entry<String, String> entry : tags.entrySet()) {
                    writer.startElement("meta", links);
                    writer.writeAttribute("name", entry.getKey(), "name");
                    writer.writeAttribute("content", entry.getValue(), "content");
                    writer.endElement("meta");
                }
                tags.clear();
            }

            for (Iterator<String> i = externalCss.iterator(); i.hasNext(); ) {
                String styleSheet = i.next();
                writer.startElement("link", links);
                writer.writeURIAttribute("href", //context.getExternalContext().getRequestContextPath() +
                        //FileStreamerPhaseListener.createResourceUrl(context, null, styleSheet), "href");
                        styleSheet, "href");
                writer.writeAttribute("rel", "stylesheet", "rel");
                writer.writeAttribute("type", "text/css", "type");
                writer.endElement("link");
            }

            for (Iterator<String> i = scripts.iterator(); i.hasNext(); ) {
                String script = i.next();
                writer.startElement("script", links);
                writer.writeAttribute("type", "text/javascript", "type");
                writer.writeURIAttribute("src", //context.getExternalContext().getRequestContextPath() +
                        //FileStreamerPhaseListener.createResourceUrl(context, null, script), "src");
                        script, "src");
                writer.endElement("script");
            }

            if (onReady.size() > 0) {
                writer.startElement("script", links);
                writer.writeAttribute("type", "text/javascript", "type");
                writer.write ("YAHOO.util.Event.onDOMReady(function() {");
                for (Iterator<String> i = onReady.iterator(); i.hasNext(); ) {
                    writer.write(i.next());
                }
                writer.write("});");
                writer.endElement("script");
                
                onReady.clear();
            }

            if (inlineCss.size() > 0) {
                writer.startElement("style", links);
                writer.writeAttribute("type", "text/css", "type");
                for (Iterator<String> i = inlineCss.iterator(); i.hasNext(); ) {
                    writer.write(i.next());
                }
                writer.endElement("style");
            }

            // Clear the lists, so the items won't be rendered again should another Links component
            // be found on the page
            scripts.clear();
            externalCss.clear();
        }
    }
}
