/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

/**
 * 
 */
package com.sun.mojarra.scales.renderer;

import com.sun.mojarra.scales.renderer.*;
import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.servlet.http.HttpServletRequest;

import com.sun.mojarra.scales.component.MultiFileUpload;

/**
 * @author Jason Lee
 *
 */
//@Renderer(@Renderer.Renders(componentFamily=MultiFileUpload.COMPONENT_FAMILY))
public class MultiFileUploadRenderer extends AbstractRenderer {
    @Override
    public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
        if (context == null) {
            throw new NullPointerException("param 'context' is null");
        }
        if (component == null) {
            throw new NullPointerException("param 'component' is null");
        }

        // suppress rendering if "rendered" property on the component is false.
        if (!component.isRendered()) {
            return;
        }

        MultiFileUpload mful = (MultiFileUpload)component;

        ResponseWriter writer = context.getResponseWriter();
        
        writer.startElement("div", mful);
        writer.startElement("div", mful);
        // For now, we're just going to support one uploader at a time on a page.  
        // If someone needs more, we'll address that later...
        writer.writeAttribute("id", "uploader", "id"); 
        writer.writeAttribute("style", "visibility: hide;", "style");
        writer.endElement("div");
//        writer.startElement("div", ul);
        writer.startElement("div", mful);
        writer.writeAttribute("id", "uploader_table", "id");
        writer.writeAttribute("style", "width: " + mful.getWidth() + 
                "; height: " + mful.getHeight() + "; border: 1px solid; float: left;", "style");
        writer.endElement("div");
        writeButton (context, mful, "Browse", "browse");
        writeButton (context, mful, "Clear", "clear");
        writeButton (context, mful, "Upload", "upload");
        writer.endElement("div");
        writer.startElement("script", mful);
        writer.writeAttribute("type", "text/javascript", "type");
        writer.write("YAHOO.util.Event.onDOMReady(function() {var uploader = new Scales.Uploader({clientId: '" + 
                mful.getClientId(context) +"', sessionId: '" + 
                ((HttpServletRequest) context.getExternalContext().getRequest()).getSession().getId() + "'});});");
        writer.endElement("script");

        // We're not using the TemplateRenderer because I can't make the following work if I do. :(
        HttpServletRequest request = (HttpServletRequest) context.getExternalContext().getRequest();
        request.getSession().setAttribute("HtmlMultiFileUpload-" + mful.getClientId(context), mful);
    }
    
    protected void writeButton(FacesContext context, MultiFileUpload component, String value, String onClick) throws IOException {
        ResponseWriter writer = context.getResponseWriter();
        writer.startElement("input", component);
        writer.writeAttribute("class", "upload_button", "class");
        writer.writeAttribute("id", "uploader_" + onClick + "_button", "id");
        writer.writeAttribute("value", value, "value");
//        writer.writeAttribute("onclick", "uploader." + onClick + "()", "onclick");
        writer.writeAttribute("type", "button", "type");
        writer.endElement("input");
        writer.startElement ("br", component);
        writer.endElement("br");
    }
    
    protected void renderJavascript(FacesContext context, MultiFileUpload component) {
        
    }
    
    /*
    protected void renderAppletTag(FacesContext context, MultiFileUpload ul) throws IOException {
        ResponseWriter writer = context.getResponseWriter();
        String template = readTemplateFile(MultiFileUpload.TEMPLATE_FILE);
        HttpServletRequest request = (HttpServletRequest) context.getExternalContext().getRequest();
        
        // Just pass back /AppContext/foo.jsf (or /AppContext/faces/foo.xhtml, etc)
        // The applet will construct the full URL based on
        // documentBase
        String uploadUrl = 
            context.getApplication().getViewHandler().getActionURL(context, generateUri(context, MultiFileUpload.UPLOAD_URI));
//            + "?" + MultiFileUpload.REQUEST_PARAM + "=" + ul.getClientId(context);
        String appletClass = ("button".equalsIgnoreCase(ul.getType())) ?
                "ButtonApplet" : "FullApplet"; // default to "full"
            
        StringBuilder deps = new StringBuilder();
        String sep = "";
            
        for (String dep : MultiFileUpload.DEP_JARS) {
            String depURI =
                context.getApplication().getViewHandler().getActionURL(context, generateUri(context, MultiFileUpload.JARS_URI + dep));
            deps.append(sep)
//                .append(ScalesUtil.generateStaticUri(dep))
                .append(depURI);
            sep = ",";
        }
        
        template = template.replaceAll("%%%DEPS%%%", deps.toString())
            .replaceAll("%%%APPLET_CLASS%%%", appletClass)
            .replaceAll("%%%WIDTH%%%", ul.getWidth())
            .replaceAll("%%%HEIGHT%%%", ul.getHeight())
            .replaceAll("%%%START_DIR%%%", ul.getStartDir())
            .replaceAll("%%%UPLOAD_URL%%%", uploadUrl)
            .replaceAll("%%%BUTTON_TEXT%%%", ul.getButtonText())
            .replaceAll("%%%FILE_FILTER%%%", ul.getFileFilter())
            .replaceAll("%%%SESSION_ID%%%", request.getSession().getId());
        writer.write(template);
        
    }
    
    protected String readTemplateFile(String fileName) {
        InputStream is = getClass().getResourceAsStream(fileName);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();

        try {
            int count = 0;
            byte[] buffer = new byte[4096];
            while ((count = is.read(buffer)) != -1) {
                if (count > 0) {
                    baos.write(buffer, 0, count);
                }
            }
            is.close();
        } catch (IOException e) {
            System.err.println(e.getMessage());
        }
        
        return baos.toString();
    }
    
    // TODO:  Look at the ScalesUtil method by the same name.  There's a good opportunity
    // for a refactor here.  Thank you, merging code bases. :)
    protected String generateUri(FacesContext context, String target) {
        String uri = "";
        String mapping = ScalesUtil.getFacesMapping(context);
        if (ScalesUtil.isPrefixMapped(mapping)) {
            uri = "/" + target + "." + mapping;
        } else {
            uri = target + mapping;
        }

        return uri;
    }
    */
}