/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

/**
 * 
 */
package com.sun.mojarra.scales.util;

import java.io.IOException;
import java.util.List;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;

import com.sun.mojarra.scales.component.MultiFileUpload;
import com.sun.mojarra.scales.model.FileHolder;


/**
 * @author Jason Lee
 *
 */
public final class MultiFileUploadPhaseListener implements PhaseListener {
    private static final long serialVersionUID = 1L;
    
    public PhaseId getPhaseId() {
        return PhaseId.RESTORE_VIEW;
    }

    public void beforePhase(PhaseEvent e) {
        // not needed
    }

    public void afterPhase(PhaseEvent e) {
        PhaseId phase = e.getPhaseId();
        if (phase == PhaseId.RESTORE_VIEW) {
            FacesContext context = FacesContext.getCurrentInstance();
            HttpServletRequest request = (HttpServletRequest) context.getExternalContext().getRequest();
            String uri = request.getRequestURI();
            if ((uri != null) && (uri.indexOf(MultiFileUpload.UPLOAD_URI) > -1)){
                HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
                String clientId = request.getParameter(MultiFileUpload.REQUEST_PARAM);
                MultiFileUpload comp = (MultiFileUpload) request.getSession().getAttribute("HtmlMultiFileUpload-" + clientId);
                request.getSession().setAttribute("HtmlMultiFileUpload-" + clientId, null); // Clean up the session

                try {
                    ServletFileUpload upload = new ServletFileUpload(new DiskFileItemFactory());

                    List items = upload.parseRequest(request);  // Get the FileItems from the request
                    FileHolder fileHolder = comp.getFileHolder();
                    if (fileHolder != null) {
                        for (int i = 0; i < items.size(); i++) {
                            FileItem item = (FileItem)items.get(i);
                            fileHolder.addFile(item.getName(), item.getInputStream());
                        }
                    }
                    String destinationUrl = comp.getDestinationUrl();
                    // If we have a complete URL, just run with that
                    if (!destinationUrl.startsWith("http")) {
                        // If we get just a view name, append the / so the API call below will be happy
                        if (destinationUrl.charAt(0) != '/') {
                            destinationUrl = "/" + destinationUrl;
                        }
                        // Generate the complete URL.
                        // NOTE:  This does *NOT* currently resolve a navigation case result string
                        // to the mapped view from the Faces configuration.  This assumes that the string
                        // in question is the *name* of a view, so "success" will become
                        // http://localhost:8080/MyContext/success.jsf, not
                        // http://localhost:8080/MyContext/youdidit.jsf that you may have mapped in
                        // your Faces config.  This is due to the lack of a portable way of making
                        // that resolution.
                        // TODO:  Fix this
                            
                        // Just pass back /AppContext/foo.jsf (or /AppContext/faces/foo.xhtml, etc)
                        // The applet will construct the full URL based on
                        // documentBase
                        destinationUrl = response.encodeURL(context.getApplication().getViewHandler().getActionURL(context, destinationUrl));
                    }
                    response.setContentType("text/text");
                    response.getOutputStream().write(destinationUrl.getBytes());
                    context.responseComplete();
                } catch (Exception e1) {
                    // TODO:  Determine the configured error page and go there?
                    e1.printStackTrace();
                    try {
                        String message = e1.getLocalizedMessage();
                        response.sendError(response.SC_INTERNAL_SERVER_ERROR, e1.getLocalizedMessage());
                    } catch (IOException ex) {
                        Logger.getLogger(MultiFileUploadPhaseListener.class.getName()).log(Level.SEVERE, null, ex);
                    }
//                        response.getOutputStream().write(("An internal error occured: " + e1.getLocalizedMessage()).getBytes());
                    context.responseComplete();
                }
            }
        }
    }
}