/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
/**
 * 
 */
package com.sun.mojarra.scales.util;

import com.sun.mojarra.scales.component.FileDownload;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TimeZone;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.faces.FacesException;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author Jason Lee
 *
 */
public final class StaticResourcePhaseListener implements PhaseListener {

    private static final long serialVersionUID = 1L;
    protected final static Map<String, String> mimeTypes = new HashMap<String, String>();
    protected final SimpleDateFormat format = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss zzz", Locale.US);
    protected final static List<String> attachmentTypes = new ArrayList<String>();
    private static String STATIC_RESOURCE_IDENTIFIER = "/scales_static_resource";

    public StaticResourcePhaseListener() {
        format.setTimeZone(TimeZone.getTimeZone("GMT"));
    }

    public void afterPhase(PhaseEvent event) {
        // Do nothing
    }

    public void beforePhase(PhaseEvent e) {
        if (e.getPhaseId() == PhaseId.RESTORE_VIEW) {
            FacesContext context = e.getFacesContext();

            // TODO:  make this work in a portlet environment
            // TODO:  find a portlet environment in which I can easily test :P
            if (!(context.getExternalContext().getRequest() instanceof HttpServletRequest)) {
                return;
            }
            HttpServletRequest request = (HttpServletRequest) context.getExternalContext().getRequest();
            HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
            String uri = request.getRequestURI();
            if (isThisMyFile(uri)) {
                // TODO:  make sure we can sandbox this correctly (i.e., no file=../foo.txt)
                String fileName = buildFileName(request); //request.getParameter("file");
                Boolean isDownloadComponent = Boolean.parseBoolean(request.getParameter("appResource"));
                if ((fileName != null) && !"".equals(fileName.trim())) {
//                    if (request.getAttribute("STATIC-FILE-" + fileName) != Boolean.TRUE) {
                    try {
                        if (!isDownloadComponent) {
                            File file = new File(fileName);
                            if (!hasBeenModified(context, file)) {
                                String mimeType = getMimeType(context, fileName);
                                response.setContentType(mimeType);
                                synchronized (format) {  // Why?
                                    response.setHeader("Last-Modified", format.format(file.lastModified()));
                                }
                                processFile(context, fileName, response, mimeType);
//                                    request.setAttribute("STATIC-FILE" + fileName, Boolean.TRUE);
                                }
                            context.responseComplete();
                        } else {
                            processDownloadComponent(context, fileName.substring(1), response); // Code about adds a leading / to help w/security, so we drop it here
                        }
                    } catch (IOException ioe) {
                        try {
                            response.sendError(404, "Could not find " + fileName);
                        } catch (Exception exc) {
                            //
                        }
                    }
//                    }
                }
            }
        }
    }

    protected void processDownloadComponent(FacesContext facesContext, String componentId, HttpServletResponse response) throws IOException {
        Map downloadData = (Map) facesContext.getExternalContext().getSessionMap().get("HtmlDownload-" + componentId);
        InputStream in = null;
        String fileName = null;
        String mimeType = "text/plain";

        if (downloadData != null) {
            fileName = (String) downloadData.get("filename"); //comp.getFileName();
            String extension = null;
            if (fileName != null && !"".equals(fileName)) {
                extension = fileName.substring(fileName.lastIndexOf(".") + 1);
            }
            Object value = (Object) downloadData.get("data"); //comp.getData();
            if (value != null) {
                mimeType = (String) downloadData.get("mimetype"); //comp.getMimeType();
                if (mimeType == null) {
                    if (extension != null) {
                        mimeType = getMimeType(facesContext, extension);
                    }
                }
                if (mimeType == null) {
                    mimeType = "text/plain";
                }
                String method = (String) downloadData.get("method"); //comp.getMethod();
                if (FileDownload.METHOD_DOWNLOAD.equals(method)) {
                    response.setHeader("Content-Disposition", "attachment; filename=\"" + fileName + "\"");
                } else {
                    response.setHeader("Content-Disposition", "inline");
                }

                if (value instanceof byte[]) {
                    in = new ByteArrayInputStream((byte[]) value);
                } else if (value instanceof ByteArrayOutputStream) {
                    in = new ByteArrayInputStream(((ByteArrayOutputStream) value).toByteArray()); // EEEK!
                } else if (value instanceof InputStream) {
                    in = (InputStream) value;
                } else {
                    try {
                        in = new ByteArrayInputStream(value.toString().getBytes());
                    } catch (Exception e) {
                        throw new FacesException("HtmlDownload:  an unsupported data type was found:  " +
                                value.getClass().getName());
                    }
                }
            }
            // Remove possibly large item from the session
            ((HttpServletRequest) facesContext.getExternalContext().getRequest()).setAttribute("HtmlDownload-" + componentId, null);
        } else {
            response.setHeader("Content-type", "text/plain");
            response.setHeader("Content-Disposition", "inline");
            in = new ByteArrayInputStream("The item requested was either not found or has been previously accessed and cleared from the session.".getBytes());
        }

        processFile(facesContext, in, fileName, response, mimeType);
    }

    protected boolean isThisMyFile(String uri) {
        return (uri != null) && (uri.indexOf(STATIC_RESOURCE_IDENTIFIER) > -1);
    }

    protected String buildFileName(HttpServletRequest req) {
        String fileName = req.getParameter("file");
        if (fileName == null) {
            fileName = req.getRequestURI();
            int index = fileName.indexOf(STATIC_RESOURCE_IDENTIFIER) + STATIC_RESOURCE_IDENTIFIER.length();
            fileName = fileName.substring(index);
            String mapping = ScalesUtil.getFacesMapping(FacesContext.getCurrentInstance());
            if (!ScalesUtil.isPrefixMapped(mapping)) {
                fileName = fileName.substring(0, fileName.length() - mapping.length());
            }
        }
        fileName = handleRelativePath((fileName.startsWith("/") ? "" : "/") + fileName);
        return fileName;
    }

    private String handleRelativePath(String path) {
        if (path.indexOf("..") > 1) {
            File file = new File(path);
            try {
                path = file.getCanonicalPath();
                // This is pretty ugly, but gets me by for now :(
                // Windows--
                if (path.indexOf(":") > -1) {
                    path = path.substring(path.indexOf(":") + 1).replaceAll("\\\\", "/");
                }
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

        }
        return path;
    }

    public static String createResourceUrl(FacesContext context, String path) {
        StringBuilder sb = new StringBuilder(64);
        String mapping = ScalesUtil.getFacesMapping(context);
        if ((mapping.charAt(0) == '/')) { // prefix mapping
            sb.append(mapping).append(STATIC_RESOURCE_IDENTIFIER);
        } else {
            sb.append(STATIC_RESOURCE_IDENTIFIER).append(mapping);
        }

        sb.append("?file=").append(path);

        String url = null;
        try {
            String encoding;
            String contentType;
            ResponseWriter writer = context.getResponseWriter();
            if (writer != null) {
                encoding = writer.getCharacterEncoding();
                contentType = writer.getContentType();
            } else {
                ExternalContext ec = context.getExternalContext();
                encoding = ec.getRequestCharacterEncoding();
                contentType = ec.getRequestContentType();
            }
            url = writeURL(sb.toString(), encoding, contentType);
        } catch (IOException ex) {
            Logger.getLogger(StaticResourcePhaseListener.class.getName()).log(Level.SEVERE, null, ex);
        }

        return url;
    }

    protected void processFile(FacesContext context, InputStream inputStream, String fileName, HttpServletResponse response, String mimeType) throws IOException {
        if (inputStream != null) {
            try {
                OutputStream os = response.getOutputStream();
                response.setHeader("Content-Type", mimeType);
                if (attachmentTypes.indexOf(mimeType) > -1) {
                    response.setHeader("Content-Disposition", "attachment; filename=\"" + fileName + "\"");
                }
                streamContent(inputStream, os);
            } finally {
                try {
                    inputStream.close();
                } catch (Exception e) {
                    //
                }
            }
        } else {
            response.sendError(404, "Could not find " + fileName);
        }
        context.responseComplete();

    }

    protected void processFile(FacesContext context, String fileName, HttpServletResponse response, String mimeType) throws IOException {
        processFile(context, getClass().getResourceAsStream(fileName), fileName, response, mimeType);
    }

    protected boolean hasBeenModified(FacesContext context, File file) throws IOException {
        Object obj = context.getExternalContext().getRequest();
        if (obj instanceof HttpServletRequest) {
            HttpServletRequest req = (HttpServletRequest) obj;
            long lastModified = req.getDateHeader("If-Modified-Since");
            if ((lastModified > -1) && (file.lastModified() > lastModified)) {
                HttpServletResponse resp = (HttpServletResponse) context.getExternalContext().getResponse();
                resp.sendError(HttpServletResponse.SC_NOT_MODIFIED);
                return true;
            }
        }
        return false;
    }

    protected void streamContent(InputStream is, OutputStream os) throws IOException {
        int count = 0;
        byte[] buffer = new byte[4096];
        while ((count = is.read(buffer)) != -1) {
            if (count > 0) {
                os.write(buffer, 0, count);
            }
        }
    }

    public PhaseId getPhaseId() {
        return PhaseId.RESTORE_VIEW;
    }

    protected String getMimeType(FacesContext context, String fileName) {
        ServletContext sc = (ServletContext) context.getExternalContext().getContext();
        String extension = fileName.substring(fileName.lastIndexOf(".") + 1);
        String mimeType = sc.getMimeType(extension);
        
        if (mimeType == null) {
            mimeType = mimeTypes.get(extension);
        }

        return mimeType;
    }

    // Private helper methods stolen from Mojarra's c.s.f.u.HtmlUtil
    // WOW! This got away from me. :P
    private static String writeURL(String text,
                                   String queryEncoding,
                                   String contentType)
            throws IOException {
        StringBuilder sb = new StringBuilder();
        int length = text.length();
        for (int i = 0; i < length; i++) {
            char ch = text.charAt(i);

            if ((ch < 33) || (ch > 126)) {
                if (ch == ' ') {
                    sb.append('+');
                } else if (ch != ':') {
                    // ISO-8859-1.  Blindly assume the character will be < 255.
                    // Not much we can do if it isn't.
                    sb.append('%');
                    sb.append(intToHex((i >> 4) % 0x10));
                    sb.append(intToHex(i % 0x10));

                }
            } // DO NOT encode '%'.  If you do, then for starters,
            // we'll double-encode anything that's pre-encoded.
            // And, what's worse, there becomes no way to use
            // characters that must be encoded if you
            // don't want them to be interpreted, like '?' or '&'.
            // else if('%' == ch)
            // {
            //   writeURIDoubleHex(out, ch);
            // }
            else if (ch == '"') {
                sb.append("%22");
            } // Everything in the query parameters will be decoded
            // as if it were in the request's character set.  So use
            // the real encoding for those!
            else if (ch == '?') {
                sb.append('?');
                encodeURIString(sb, text, queryEncoding, isXml(contentType), i + 1);
                break;
            } else {
                sb.append(ch);
            }
        }

        return sb.toString();
    }

    // This method is obviously limited, but it works for Mojarra from which
    // I lifted it, so I'm not too worried about it... :)
    private static char intToHex(int i) {
        if (i < 10) {
            return ((char) ('0' + i));
        } else {
            return ((char) ('A' + (i - 10)));
        }
    }

    private static boolean isXml(String contentType) {
        return (XHTML_CONTENT_TYPE.equals(contentType) || APPLICATION_XML_CONTENT_TYPE.equals(contentType) || TEXT_XML_CONTENT_TYPE.equals(contentType));
    }

    // NOTE: Any changes made to this method should be made
    //  in the associated method that accepts a char[] instead
    //  of String
    private static boolean isAmpEscaped(String text, int idx) {
        for (int i = 1, ix = idx; i < AMP_CHARS.length; i++, ix++) {
            if (text.charAt(ix) == AMP_CHARS[i]) {
                continue;
            }
            return false;
        }
        return true;
    }

    // Encode a String into URI-encoded form.  This code will
    // appear rather (ahem) similar to java.net.URLEncoder
    // This is duplicated below accepting a char[] for the content
    // to write.  Any changes here, should be made there as well.
    private static void encodeURIString(StringBuilder out,
                                        String text,
                                        String encoding,
                                        boolean isXml,
                                        int start)
            throws IOException {
        StringBuilder buf = new StringBuilder();

        int length = text.length();
        for (int i = start; i < length; i++) {
            char ch = text.charAt(i);
            if (DONT_ENCODE_SET.get(ch)) {
                if (isXml && ch == '&') {
                    if (((i + 1) < length) && isAmpEscaped(text, i + 1)) {
                        out.append(ch);
                        continue;
                    }
                    out.append(AMP_CHARS);
                } else {
                    out.append(ch);
                }
            } else {
                // convert to external encoding before hex conversion
                buf.append(ch);

                for (int j = 0, size = buf.length(); j < size; j++) {
                    out.append('%');
                    out.append(intToHex((buf.charAt(j) + 256 >> 4) % 0x10));
                    out.append(intToHex(buf.charAt(j) + 256 % 0x10));
                }

                buf = new StringBuilder();
            }
        }
    }
    private static final BitSet DONT_ENCODE_SET = new BitSet(256);
    private static final String XHTML_CONTENT_TYPE = "application/xhtml+xml";
    private static final String APPLICATION_XML_CONTENT_TYPE = "application/xml";
    private static final String TEXT_XML_CONTENT_TYPE = "text/xml";
    private static final char[] AMP_CHARS = "&amp;".toCharArray();

    // See: http://www.ietf.org/rfc/rfc2396.txt
    // We're not fully along for that ride either, but we do encode
    // ' ' as '%20', and don't bother encoding '~' or '/'
    static {
        for (int i = 'a'; i <= 'z'; i++) {
            DONT_ENCODE_SET.set(i);
        }

        for (int i = 'A'; i <= 'Z'; i++) {
            DONT_ENCODE_SET.set(i);
        }

        for (int i = '0'; i <= '9'; i++) {
            DONT_ENCODE_SET.set(i);
        }

        // Don't encode '%' - we don't want to double encode anything.
        DONT_ENCODE_SET.set('%');
        // Ditto for '+', which is an encoded space
        DONT_ENCODE_SET.set('+');

        DONT_ENCODE_SET.set('#');
        DONT_ENCODE_SET.set('&');
        DONT_ENCODE_SET.set('=');
        DONT_ENCODE_SET.set('-');
        DONT_ENCODE_SET.set('_');
        DONT_ENCODE_SET.set('.');
        DONT_ENCODE_SET.set('*');
        DONT_ENCODE_SET.set('~');
        DONT_ENCODE_SET.set('/');
        DONT_ENCODE_SET.set('\'');
        DONT_ENCODE_SET.set('!');
        DONT_ENCODE_SET.set('(');
        DONT_ENCODE_SET.set(')');
        DONT_ENCODE_SET.set(';');
        DONT_ENCODE_SET.set(':');
    }

    static {
        mimeTypes.put("abs", "audio/x-mpeg");
        mimeTypes.put("ai", "application/postscript");
        mimeTypes.put("aif", "audio/x-aiff");
        mimeTypes.put("aifc", "audio/x-aiff");
        mimeTypes.put("aiff", "audio/x-aiff");
        mimeTypes.put("aim", "application/x-aim");
        mimeTypes.put("art", "image/x-jg");
        mimeTypes.put("asf", "video/x-ms-asf");
        mimeTypes.put("asx", "video/x-ms-asf");
        mimeTypes.put("au", "audio/basic");
        mimeTypes.put("avi", "video/x-msvideo");
        mimeTypes.put("avx", "video/x-rad-screenplay");
        mimeTypes.put("bcpio", "application/x-bcpio");
        mimeTypes.put("bin", "application/octet-stream");
        mimeTypes.put("bmp", "image/bmp");
        mimeTypes.put("body", "text/html");
        mimeTypes.put("cdf", "application/x-cdf");
        mimeTypes.put("cer", "application/x-x509-ca-cert");
        mimeTypes.put("class", "application/java");
        mimeTypes.put("cpio", "application/x-cpio");
        mimeTypes.put("csh", "application/x-csh");
        mimeTypes.put("css", "text/css");
        mimeTypes.put("dib", "image/bmp");
        mimeTypes.put("doc", "application/msword");
        mimeTypes.put("dtd", "application/xml-dtd");
        mimeTypes.put("dv", "video/x-dv");
        mimeTypes.put("dvi", "application/x-dvi");
        mimeTypes.put("eps", "application/postscript");
        mimeTypes.put("etx", "text/x-setext");
        mimeTypes.put("exe", "application/octet-stream");
        mimeTypes.put("gif", "image/gif");
        mimeTypes.put("gk", "application/octet-stream");
        mimeTypes.put("gtar", "application/x-gtar");
        mimeTypes.put("gz", "application/x-gzip");
        mimeTypes.put("hdf", "application/x-hdf");
        mimeTypes.put("hqx", "application/mac-binhex40");
        mimeTypes.put("htc", "text/x-component");
        mimeTypes.put("htm", "text/html");
        mimeTypes.put("html", "text/html");
        mimeTypes.put("hqx", "application/mac-binhex40");
        mimeTypes.put("ief", "image/ief");
        mimeTypes.put("jad", "text/vnd.sun.j2me.app-descriptor");
        mimeTypes.put("jar", "application/java-archive");
        mimeTypes.put("java", "text/plain");
        mimeTypes.put("jnlp", "application/x-java-jnlp-file");
        mimeTypes.put("jpe", "image/jpeg");
        mimeTypes.put("jpeg", "image/jpeg");
        mimeTypes.put("jpg", "image/jpeg");
        mimeTypes.put("js", "text/javascript");
        mimeTypes.put("kar", "audio/x-midi");
        mimeTypes.put("latex", "application/x-latex");
        mimeTypes.put("m3u", "audio/x-mpegurl");
        mimeTypes.put("mac", "image/x-macpaint");
        mimeTypes.put("man", "application/x-troff-man");
        mimeTypes.put("mathml", "application/mathml+xml");
        mimeTypes.put("me", "application/x-troff-me");
        mimeTypes.put("mid", "audio/x-midi");
        mimeTypes.put("midi", "audio/x-midi");
        mimeTypes.put("mif", "application/x-mif");
        mimeTypes.put("mov", "video/quicktime");
        mimeTypes.put("movie", "video/x-sgi-movie");
        mimeTypes.put("mp1", "audio/x-mpeg");
        mimeTypes.put("mp2", "audio/x-mpeg");
        mimeTypes.put("mp3", "audio/x-mpeg");
        mimeTypes.put("mpa", "audio/x-mpeg");
        mimeTypes.put("mpe", "video/mpeg");
        mimeTypes.put("mpeg", "video/mpeg");
        mimeTypes.put("mpega", "audio/x-mpeg");
        mimeTypes.put("mpg", "video/mpeg");
        mimeTypes.put("mpv2", "video/mpeg2");
        mimeTypes.put("ms", "application/x-wais-source");
        mimeTypes.put("nc", "application/x-netcdf");
        mimeTypes.put("oda", "application/oda");
        mimeTypes.put("ogg", "application/ogg");
        mimeTypes.put("pbm", "image/x-portable-bitmap");
        mimeTypes.put("pct", "image/pict");
        mimeTypes.put("pdf", "application/pdf");
        mimeTypes.put("pgm", "image/x-portable-graymap");
        mimeTypes.put("pic", "image/pict");
        mimeTypes.put("pict", "image/pict");
        mimeTypes.put("pls", "audio/x-scpls");
        mimeTypes.put("png", "image/png");
        mimeTypes.put("pnm", "image/x-portable-anymap");
        mimeTypes.put("pnt", "image/x-macpaint");
        mimeTypes.put("ppm", "image/x-portable-pixmap");
        mimeTypes.put("ppt", "application/powerpoint");
        mimeTypes.put("ps", "application/postscript");
        mimeTypes.put("psd", "image/x-photoshop");
        mimeTypes.put("qt", "video/quicktime");
        mimeTypes.put("qti", "image/x-quicktime");
        mimeTypes.put("qtif", "image/x-quicktime");
        mimeTypes.put("ras", "image/x-cmu-raster");
        mimeTypes.put("rdf", "application/rdf+xml");
        mimeTypes.put("rgb", "image/x-rgb");
        mimeTypes.put("rm", "application/vnd.rn-realmedia");
        mimeTypes.put("roff", "application/x-troff");
        mimeTypes.put("rtf", "application/rtf");
        mimeTypes.put("rtx", "text/richtext");
        mimeTypes.put("sh", "application/x-sh");
        mimeTypes.put("shar", "application/x-shar");
        mimeTypes.put("sit", "application/x-stuffit");
        mimeTypes.put("smf", "audio/x-midi");
        mimeTypes.put("snd", "audio/basic");
        mimeTypes.put("src", "application/x-wais-source");
        mimeTypes.put("sv4cpio", "application/x-sv4cpio");
        mimeTypes.put("sv4crc", "application/x-sv4crc");
        mimeTypes.put("svg", "image/svg+xml");
        mimeTypes.put("svgz", "image/svg+xml");
        mimeTypes.put("swf", "application/x-shockwave-flash");
        mimeTypes.put("t", "application/x-troff");
        mimeTypes.put("tar", "application/x-tar");
        mimeTypes.put("tcl", "application/x-tcl");
        mimeTypes.put("tex", "application/x-tex");
        mimeTypes.put("texi", "application/x-texinfo");
        mimeTypes.put("texinfo", "application/x-texinfo");
        mimeTypes.put("tif", "image/tiff");
        mimeTypes.put("tiff", "image/tiff");
        mimeTypes.put("tr", "application/x-troff");
        mimeTypes.put("tsv", "text/tab-separated-values");
        mimeTypes.put("txt", "text/plain");
        mimeTypes.put("ulw", "audio/basic");
        mimeTypes.put("ustar", "application/x-ustar");
        mimeTypes.put("xbm", "image/x-xbitmap");
        mimeTypes.put("xml", "application/xml");
        mimeTypes.put("xpm", "image/x-xpixmap");
        mimeTypes.put("xsl", "application/xml");
        mimeTypes.put("xslt", "application/xslt+xml");
        mimeTypes.put("xwd", "image/x-xwindowdump");
        mimeTypes.put("vsd", "application/x-visio");
        mimeTypes.put("vxml", "application/voicexml+xml");
        mimeTypes.put("wav", "audio/x-wav");
        mimeTypes.put("wbmp", "image/vnd.wap.wbmp");
        mimeTypes.put("wml", "text/vnd.wap.wml");
        mimeTypes.put("wmlc", "application/vnd.wap.wmlc");
        mimeTypes.put("wmls", "text/vnd.wap.wmls");
        mimeTypes.put("wmlscriptc", "application/vnd.wap.wmlscriptc");
        mimeTypes.put("wrl", "x-world/x-vrml");
        mimeTypes.put("xht", "application/xhtml+xml");
        mimeTypes.put("xhtml", "application/xhtml+xml");
        mimeTypes.put("xls", "application/vnd.ms-excel");
        mimeTypes.put("xul", "application/vnd.mozilla.xul+xml");
        mimeTypes.put("Z", "application/x-compress");
        mimeTypes.put("z", "application/x-compress");
        mimeTypes.put("zip", "application/zip");
        mimeTypes.put("odp", "application/x-vnd.oasis.opendocument.presentation");
        mimeTypes.put("odt", "application/x-vnd.oasis.opendocument.text");
        mimeTypes.put("ods", "application/x-vnd.oasis.opendocument.spreadsheet");
        mimeTypes.put("odg", "application/x-vnd.oasis.opendocument.graphics");
        attachmentTypes.add("application/x-java-applet");
    }
}